#include <time.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/times.h>
#include <unistd.h>
#include <stdio.h>
#include "shared.h"
#include "serverlib.h"

const char * DATEFORMAT = "%A %d %B %Y @ %H:%M %Z"; /*Date format*/
const char * TIMEELAPSEDFORMAT = "%H:%M"; /*string format for time elapsed*/
#define CHARSIZE sizeof(char)
const size_t MEMORYSEGMENTSIZE = MAXREPLYSIZE * CHARSIZE; /*Memory allocation size*/
clock_t intialization_clockT = 0; /*clock ticks at initialization time*/
volatile long ticksPerSecond = -1; /*ticks per second*/

/*
function getCurrentTime
task:  returns the system time.
input: none
output: a zero-terminated string on success. The string should be cleared with free(). NULL on error
*/
char * getCurrentTime(){
  time_t currTime_timeT = time(NULL);
  struct tm * formattedTime_ptm = localtime(&currTime_timeT);
  char * returnVal = malloc(MEMORYSEGMENTSIZE);

  if (returnVal != NULL){
    if(strftime(returnVal,MAXREPLYSIZE, DATEFORMAT,formattedTime_ptm) !=0) return returnVal;
  }
  /*If this is executed, we have an error on memory allocation or in time creation, so try a fallback and generate error message*/
  WriteError("getCurrentTime(): Unable to allocate memory or segment size too small\n");

  return ctime(&currTime_timeT); /*It has been tested that we can call free() on this with no error*/
}

/*
function initializeClock
task: Initializes the clock, in order to have accurate 'time elapsed' value
input: none
output: none
*/
void initializeClock(){
  intialization_clockT = times(NULL);
  ticksPerSecond = sysconf(_SC_CLK_TCK);
}
/*
function getTimeElapsed
task: returns the time elapsed since initializeClock() was called, or calls it it it wasn't done previously
input: none
output: zero-terminated string. This string can be cleared with free(). NULL is returned on error
*/
char * getTimeElapsed(){
  clock_t elapsedTime_clockT;
  char * returnVal = NULL;
  if( ticksPerSecond == -1) initializeClock();
  elapsedTime_clockT = times(NULL) -intialization_clockT;

  if (elapsedTime_clockT >= 0){ /*if negative, we have an error*/
     returnVal= malloc(MEMORYSEGMENTSIZE);
     if(returnVal != NULL){/*Write only if segment is OK*/
       sprintf(returnVal, "%.2f seconds", (double)elapsedTime_clockT / ticksPerSecond);
     }
  }

  return returnVal;
}


#ifdef TIMEOUT_TEST

#include <stdio.h>
int main(int argc, char** argv){

  char * val;
  int i;

  initializeClock();

  val = getCurrentTime();
  printf("%s\n", val);
  free (val);

  for (i =0; i < 100000000; i++){

  }

  val = getTimeElapsed();
  printf("%s\n", val);
  free (val);

  return 0;
}

#endif

