
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <fcntl.h>
#include <signal.h>
   /*
   WriteError
   task: print a string to standard error
   input: toReport_sz: string to display
   output: -1 on error, 0 otherwise
   */
int WriteError(char * toReport_sz){
  int sizeString_int = strlen(toReport_sz);

  if (write(STDERR_FILENO,toReport_sz, sizeString_int) != sizeString_int) return -1;

  return 0;

}


int ensureStandardOutputAtomicity(){
  struct stat stdOut_stat;
  int result_int;
  int currState_int;

  /*extract information on standard input*/
  result_int = fstat(STDOUT_FILENO, &stdOut_stat);
  if (result_int == -1) return -1;

  if(!S_ISFIFO(stdOut_stat.st_mode)){ /*If not FIFO or pipe, we must ensure O_APPEND is set*/
    currState_int = fcntl(STDOUT_FILENO, F_GETFL); /*Extract file descriptor settings*/
    if (currState_int == -1) return -1;
    if (! (currState_int & O_APPEND)) /*if not having the O_APPEND mode bit set*/
       currState_int = fcntl(STDOUT_FILENO, F_SETFL, currState_int | O_APPEND); /*Set O_APPEND*/
    if(currState_int == -1) return -1;
  }
  return 0;
}


void sleepALittle(long milliseconds){
  struct timeval wait_timeval;

  if (milliseconds <=0) return;
  wait_timeval.tv_sec = 0;
  wait_timeval.tv_usec = milliseconds * 1000;
  select(0,NULL,NULL,NULL,&wait_timeval);/*Sleep for a while*/
}

/*
Sets handler for terminating signals
*/
int setSignalHandlerForTerminatingSignals(struct sigaction settings_sigaction){
	int result = 0;

	result += sigaction(SIGABRT,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGBUS,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGFPE,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGHUP,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGILL,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGINT,&settings_sigaction, NULL ); /*Load handler*/
	result += sigaction(SIGPIPE,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGQUIT,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGSEGV,&settings_sigaction, NULL );/*load handler*/
	result += sigaction(SIGTERM,&settings_sigaction, NULL );/*load handler*/

	if(result != 0) return -1;/*If nonzero, then one has yielded an error*/

	return 0;
}






