#ifndef TIMERSERVER_API_H
#define TIMERSERVER_API_H

/* This API is meant to allow a standardized server code, which will have a different set of communication
   methods with clients. Different implementations in separate C files will differientiate
   between the TCP and the IPC versions.

   API specification
*/

#include "shared.h"

/*
function pollForRequest
task: fetch the first pending request, returns only on a request or error
input: none
output: a pointer to a struct timeRequest on success, NULL on error
*/
struct timeRequest  * pollForRequest();

/*
function sendReply
task: send a reply to client identified by clientID_timeRequest
input: clientID_timeRequest: struct timeRequest: time request holding client information
          reply: zero-terminated string: reply to send to the client
output: -1 on error, 0 otherwise
*/
int sendReply(struct timeRequest clientID_timeRequest, const char * reply);

/*
function initializeServer
task: initialize communication mechanisms
input: none
output: -1 on error, 0 otherwise
*/
int initializeServer();

/*
function hasConnectedClients
task: tells whether teh server should keep running to server clients
input: none
output: for IPC: TRUE
            for TCP: TRUE if no clients have ever connected or if there are still active clients, FALSE otherwise
*/
boolean hasConnectedClients();

/*
function disconnectClient
task: for socket-version only. Disconnects a client
input: none
output: -1 on error, 0 otherwise
*/
int disconnectClient(int clientID_int);
/*
function shutDown
task: terminate communication mechanisms
input: none
output: -1 on error, 0 otherwise
*/
int shutDown();

#endif


