/* This API is meant to allow a standardized server code, which will have a different set of communication
   methods with clients. Different implementations in separate C files will differientiate
   between the TCP and the IPC versions.

   IPC Implementation

   NOTE: Tracking of connected clients is yet to be implemented

*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include "../common/shared.h" /*for boolean*/
#include "../common/timeserver_api.h" /*For function definitions*/
#include "../common/serverlib.h"
#include "timeserver_ipc.h" /*For what is being indirected to*/


extern int semaphoreID;
extern int sharedMemoryID;
/*extern struct timeRequest * request_ptimeRequest;*/

/*
function pollForRequest
task: fetch the first pending request, returns only on a request or error
input: none
output: a pointer to a struct timeRequest on success, NULL on error
*/
struct timeRequest  * pollForRequest(){
  boolean waiting_bool = TRUE;
  struct timeRequest  * returnVal_ptimeRequest;

  while(waiting_bool){
  	notifyServerReady();
  	clientHasConnected();
  	if (clientPutRequest()){
	  waiting_bool = FALSE;

	  returnVal_ptimeRequest = getRequest();
        }else sleepALittle(100);
  }

  return returnVal_ptimeRequest;
}

/*
function initializeServer
task: initialize communication mechanisms
input: none
output: -1 on error, 0 otherwise
*/
int initializeServer(){

  /*Set signal handling*/
  if (InitSignalHandler() == -1) return -1; /*Error on initializing the handler*/

  /*Initialize clock for time elapsed feature*/
  initializeClock();

  /*set IPC*/
  if (initializeServerSetIPC() == -1) return -1;

  return 0;
}


/*
function hasConnectedClients
task: tells whether teh server should keep running to server clients
input: none
output: for IPC: TRUE
            for TCP: TRUE if no clients have ever connected or if there are still active clients, FALSE otherwise
*/
boolean hasConnectedClients(){
  return TRUE;
}

/*
function shutDown
task: terminate communication mechanisms
input: none
output: -1 on error, 0 otherwise
*/
int shutDown(){
  cleanupServerSetIPC();
  return 0;
}

/*
function sendReply
task: send a reply to client identified by clientID_timeRequest
input: clientID_timeRequest: struct timeRequest: time request holding client information
          reply: zero-terminated string: reply to send to the client
output: -1 on error, 0 otherwise
*/
int sendReply(struct timeRequest clientID_timeRequest, const char * reply_sz){
  int clientSemaphoreID_int = clientID_timeRequest.replyNotificationSemaphore_int;
  int clientSharedMemID_int= clientID_timeRequest.replyMemorySegmentID_int;
  char * clientSharedMem_sz;
  int result_int = 0;
 struct sembuf notify_sembuf;

  /*========= attach and put reply in shared memory*/
  clientSharedMem_sz = shmat(clientSharedMemID_int, NULL, 0);
  if (clientSharedMem_sz != (char *) -1){
     memcpy(clientSharedMem_sz, reply_sz, strlen(reply_sz)); /*Do not copy on error*/
  }
  else{
    WriteError("Cannot attach shared memory segment to reply to client\n");
  }

  result_int += shmdt(clientSharedMem_sz);
  if (result_int != 0) return -1;

  /*=========== notify client*/
  notify_sembuf.sem_num =0;
  notify_sembuf.sem_op = +1;
  notify_sembuf.sem_flg = 0;
  if (semop(clientSemaphoreID_int, &notify_sembuf, 1) == -1) {
    WriteError("Cannot perform semaphore operation to notify client of completion\n");
    return -1;
  }

  return 0;
}

/*
function disconnectClient
task: for socket-version only. Disconnects a client
input: none
output: -1 on error, 0 otherwise
*/
int disconnectClient(int clientID_int){
  return 0;
}

