#ifndef TIMESERVER_IPC_H
#define TIMESERVER_IPC_H

#include "../common/shared.h"
/*
function InitSignalHandler
task: Sets the signal handler to catch all killing signals
         The signal handler is meant as to ensure IPC stuctures are cleaned on termination
input: none
output: -1 on error, 0 otherwise
*/
int InitSignalHandler();

/*
function initializedServerSetiPC
task: Initializes IPC structures
         The shared memory segment is mapped to SERVER_SHARED_MEMORY_KEY
	 The semaphore set is mapped to SERVER_SEMAPHORES_KEY
input; none
output: -1 on error, 0 otherwise
*/
int initializeServerSetIPC();

/*
function cleanupServerSetIPC
task: Destroys IPC structures
input:none
output: none
*/
void cleanupServerSetIPC();

/*
function notifyServerReady
task: Sets semaphore SERVER_READY indicating server is ready
input: none
output: -1 on error, 0 otherwise
*/
int notifyServerReady();

/*
function clientPutRequest
task: monitor semaphore REQUEST_FORWARDED_NOTIFICATION for some time to determine if client has put request in or not
input: none
output: returns TRUE when request is in, FALSE when request timed out
*/
boolean clientPutRequest();

/*
function clientHasConnected
task: monitor semaphore SERVER_READY for client to obtain it
input: none
output: TRUE when client has connected
*/
boolean clientHasConnected();


/*
function getRequest
task: obtain a request from the shared memory segment
input: none
return: return a pointer to a struct timeRequest holding the current request, NULL on error
*/
struct timeRequest * getRequest();

#endif


