#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "./common/shared.h"
#include "./common/serverlib.h"
#include "./common/timeserver_api.h"

int main(int argc, char** argv){
  struct timeRequest  * received_pTimeRequest;
  char * requestReply_sz;
  const char * INITIALIZE_MESSAGE ="Time server initializing\n";
  boolean sending_reply_boolean = FALSE;
  boolean freeingMem_boolean;
  #ifdef SERVER_MAIN_PRINTINFO
  char messageOutBuffer_sz[1024];
  char *errorStatusText_string;
  char *requestName_string;
  #endif


  if (ensureStandardOutputAtomicity() == -1){
    WriteError("Error in initializing output\n");
    exit(1);
  }

  /*initialize communication mechanisms*/
  write(STDOUT_FILENO,INITIALIZE_MESSAGE, strlen(INITIALIZE_MESSAGE));
  if (initializeServer() == -1){
    WriteError("Unable to initialize server\n");
    exit(1);
  }

  do{
    #ifdef SERVER_MAIN_PRINTINFO
    errorStatusText_string = "no error";
    #endif

    /*Receive request*/
    if ((received_pTimeRequest = pollForRequest()) == NULL) {
    #ifdef SERVER_MAIN_PRINTINFO
      WriteError("Critical error: request polling returned an error\n");
    #endif
      break;
    }
   sending_reply_boolean = TRUE;
   freeingMem_boolean = TRUE;

    switch(received_pTimeRequest->reqType){
      case TIME:
      #ifdef SERVER_MAIN_PRINTINFO
        requestName_string ="current time";
      #endif
        requestReply_sz = getCurrentTime();
      break;
      case TIME_ELAPSED:
      #ifdef SERVER_MAIN_PRINTINFO
        requestName_string ="time elapsed";
      #endif
        requestReply_sz = getTimeElapsed();
      break;
      case DISCONNECT:
        sending_reply_boolean = FALSE;
	freeingMem_boolean = FALSE;
	disconnectClient(received_pTimeRequest->clientID);
      #ifdef SERVER_MAIN_PRINTINFO
	requestName_string = "disconnection";
	requestReply_sz = "acknowledgement. The client was disconnected.";
      #endif
      break;
      default:
        requestReply_sz = "Invalid Request";
	freeingMem_boolean = FALSE;
    }

    if (requestReply_sz == NULL) {
    #ifdef SERVER_MAIN_PRINTINFO
      WriteError("Error on acquiring time information\n");
    #endif

      break;
    }
    if (sending_reply_boolean){
	if (sendReply(*received_pTimeRequest ,requestReply_sz) == -1){
	#ifdef SERVER_MAIN_PRINTINFO
	WriteError("Error on replying to client\n");
	errorStatusText_string = "error on sending reply to client";
	#endif
	}
    }

    #ifdef SERVER_MAIN_PRINTINFO
    if (freeingMem_boolean)
    sprintf(messageOutBuffer_sz,
              "Reading from client %.0f for the %s was replied with %s\nReply was: %s\n",
              (float)received_pTimeRequest->clientID,
	      requestName_string,
	      errorStatusText_string, 
	      requestReply_sz);

    else
    sprintf(messageOutBuffer_sz,
              "Reading from client %.0f for the %s was replied with %s\n",
              (float)received_pTimeRequest->clientID,
	      requestName_string,
	      errorStatusText_string);


    write(STDOUT_FILENO,messageOutBuffer_sz, strlen(messageOutBuffer_sz));

    #endif
  if (freeingMem_boolean){
    free(requestReply_sz);
  }
  }while(hasConnectedClients());

  #ifdef SERVER_MAIN_PRINTINFO
  WriteError("Time server shutting down\n");
  #endif

  shutDown();
  return 0;
}


