/* This API is meant to allow a standardized server code, which will have a different set of communication
   methods with clients. Different implementations in separate C files will differientiate
   between the TCP and the IPC versions.

   TCP Implementation
*/

#include "../common/shared.h" /*for boolean*/
#include "../common/timeserver_api.h" /*For function definitions*/
#include "../common/serverlib.h"
#include "timeserver_tcp.h"
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/*
function pollForRequest
task: fetch the first pending request, returns only on a request or error
input: none
output: a pointer to a struct timeRequest on success, NULL on error
*/
struct timeRequest  * pollForRequest(){
  struct timeRequest * returnVal_ptimeRequest = NULL;

  /*WriteError("Polling for a request\n");*/

  while(returnVal_ptimeRequest == NULL && servicing()){
     if (AcceptNewConnection() == -1) return NULL;
     returnVal_ptimeRequest = pollRequest();
     if (errno != 0) return NULL; /*if errno is set, we have an error*/
     sleepALittle(250);
  }
  /*WriteError("Request polled\n");*/
  return returnVal_ptimeRequest;
}

/*
function sendReply
task: send a reply to client identified by clientID_timeRequest
input: clientID_timeRequest: struct timeRequest: time request holding client information
          reply: zero-terminated string: reply to send to the client
output: -1 on error, 0 otherwise
*/
int sendReply(struct timeRequest clientID_timeRequest, const char * reply){
  if (forwardReply(reply) == -1) return -1;
  return 0;
}

/*
function initializeServer
task: initialize communication mechanisms
input: none
output: -1 on error, 0 otherwise
*/
int initializeServer(){
  pid_t daemon_pidT;
    /*Initialize clock for time elapsed feature*/
  initializeClock();

  /*Setting as daemon. From Stevens p. 418*/
  if (( daemon_pidT = fork()) <0 ){
    WriteError("Cannot fork()\n");
    return -1; /*Error on forking*/
  }
  else if (daemon_pidT != 0) exit(0);
  setsid();
  chdir("/");
  umask(0);

   /*WriteError("Setting Connection\n");*/
   return setConnection();
}

/*
function hasConnectedClients
task: tells whether teh server should keep running to server clients
input: none
output: for IPC: TRUE
            for TCP: TRUE if no clients have ever connected or if there are still active clients, FALSE otherwise
*/
boolean hasConnectedClients(){
  return servicing();
}

/*
function shutDown
task: terminate communication mechanisms
input: none
output: -1 on error, 0 otherwise
*/
int shutDown(){
  /*WriteError("Shutting down\n");*/
  shutDownNetwork();
  return 0;
}

/*
function disconnectClient
task: for socket-version only. Disconnects a client
input: none
output: -1 on error, 0 otherwise
*/
int disconnectClient(int clientID_int){
  struct timeRequest empty;
  int result = 0;


  /*WriteError("Disconnecting client\n");*/

/*Reply to client*/
 result += sendReply(empty, "acknowledged");
 result += removeClient(clientID_int);

 if (result != 0) return -1;
 else return 0;
}


