#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/poll.h>
#include <poll.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>


#include "../common/shared.h"
#include "timeserver_tcp.h"
#include "tcp_common.h"

/*===============Internal data structure  to keep track of connected clients ======*/
int currentConnections = 0; /*current total of connections*/
#define MAXCONNECTIONS 50
int clientFileDescriptor_aint[MAXCONNECTIONS]; /*list of file descriptors of sockets of connections*/
/*
function canHandleMoreClients
task: tells wheter or not maximum capacity has been reached
input: none
output: TRUE if more clients can be connected, FALSE otherwise
*/
boolean canHandleMoreClients(){
  if (currentConnections < MAXCONNECTIONS) return TRUE;
  else return FALSE;
}
/*
function hasClientsConnected
task: tells wheter clients are currently held in the list
input:none
output: TRUE if clients are in the list, FALSE, otherwise
*/
boolean hasClientsConnected(){
 if (currentConnections > 0) return TRUE;
 else return FALSE;
}

/*
function addClient
task: add a connected client's file descriptor to the list. There is no collision detection performed
input: fileDescriptorToAdd_int: int: file descriptor of the client to add in the list
output: -1 on error, 0 otherwise
*/
int addClient(int fileDescriptorToAdd_int){
  if (currentConnections >=MAXCONNECTIONS) return -1;
  clientFileDescriptor_aint[currentConnections] = fileDescriptorToAdd_int;
  currentConnections++;
  return 0;
}
/*
function removeClient
task: remove a client file descriptor from the list
input: fileDescriptorToRemove_int: int: file descriptor of the client to remove from the list
output: -1 on error, 0 otherwise
*/
int removeClient(int fileDescriptorToRemove_int){
  int i;
  int numBytesToMove_int;

  for (i = 0; i <currentConnections; i++){
    if (clientFileDescriptor_aint[i] ==fileDescriptorToRemove_int){
      numBytesToMove_int = (currentConnections - i -1) * sizeof(int);
      if(numBytesToMove_int>0){
        /*shift all the rest of the data to the left*/
        memcpy(clientFileDescriptor_aint+i, clientFileDescriptor_aint+i+1,numBytesToMove_int);
      }
      /*Decrement connections*/
      currentConnections --;
      break;
    }
  }
  return 0;
}

/*========================Connection Management ================*/
/*=========== Data ==================*/
/*TRUE if a client has connected before*/
boolean previousConnections = FALSE;

/*File descriptor of accepting socket*/
int acceptingSocket_int = -1;

struct timeRequest current_timeRequest; /*Time request of reference*/

/*========== Public methods==================*/
/*
function setConnection
task: initialize server socket
input:none
output: -1 on error, 0 otherwise
*/
int setConnection(){
  struct sockaddr_in bindingAddress_sockaddrIn;

  /*Create socket*/
  acceptingSocket_int = socket(AF_INET, SOCK_STREAM,0);
  if (acceptingSocket_int < 0) return -1;

   /*Prepare bind-specific information From Donahoo/Calvert p.19*/
  memset(&bindingAddress_sockaddrIn, 0, sizeof(struct sockaddr_in));
  bindingAddress_sockaddrIn.sin_family = AF_INET;
  bindingAddress_sockaddrIn.sin_addr.s_addr = htonl(INADDR_ANY);
  bindingAddress_sockaddrIn.sin_port = htons(SERVERPORT);


  /*bind socket*/
  if (bind(acceptingSocket_int, (struct sockaddr *) &bindingAddress_sockaddrIn, sizeof(bindingAddress_sockaddrIn)) <0){
    return -1;
  }

  /*register accepting connections*/
  if (listen(acceptingSocket_int,MAXCONNECTIONS) <0) return -1;

  return 0;
}

/*
function removeClient
task: disconnect socket and remove client from the list of connected clients
input: fileDescriptorToRemove_int: int: file descriptor of the client to disconnect
output: -1 on error, 0 otherwise
*/
int freeDisconnectedClient(int clientFileDescriptor_int){
  close(clientFileDescriptor_int);
  removeClient(clientFileDescriptor_int);
  return 0;
}


/*
function AcceptNewConnection
task: poll listening socket for an incoming connection and accept it if there is one
input: none
output: -1 on error, 0 otherwise. 0 is returned wheter a client has been added or not.
*/
int AcceptNewConnection(){
  struct pollfd socket_pollfd;
  int newConnectionSocket_int;

  if (!canHandleMoreClients()) return 0;

  if (acceptingSocket_int == -1) return -1;

  if (setSocketParams(acceptingSocket_int) == -1) return -1;

  socket_pollfd.fd =acceptingSocket_int;
  socket_pollfd.events = POLLIN;

  if (poll(&socket_pollfd, 1,0) == -1) return -1;

  if (socket_pollfd.revents & POLLIN){ /*Data is in*/
    newConnectionSocket_int = accept(acceptingSocket_int, NULL, NULL);
    if (newConnectionSocket_int == -1) return -1;
    addClient(newConnectionSocket_int);
    previousConnections = TRUE;
    /*WriteError("New connection accepted\n");*/
  }

  return 0;
}
/*
function pollRequest
task: find the first socket in the internal list with a request in it. If a client has disconnected, it is removed from the list
input: none
output: a pointer to a struct timeRequest * if a request is in, NULL otherwise.
Furthermore, errno is set to the following:
  0: success
  EIO: error on polling on the socket
*/
struct timeRequest * pollRequest(){
  int i;
  struct timeRequest * returnVal_ptimeRequest = NULL;
  struct pollfd socket_pollfd;
  int sizeRequest_int = sizeof(struct timeRequest);

  socket_pollfd.events = POLLIN;

  /*WriteError("pollRequest()\n");*/


/*traverse the list, poll, until one has data in it*/
  for (i = 0; i <currentConnections; i++){
    socket_pollfd.fd =clientFileDescriptor_aint[i];
    /*poll descriptor*/
    if (poll(&socket_pollfd, 1,0) == -1) {
      errno = EIO;
      return NULL;
    }

    /*====================== If data received=================*/
    if (socket_pollfd.revents & POLLIN){ /*Data is in*/
      returnVal_ptimeRequest = &current_timeRequest;

      if (/* client disconnected*/
          recv(clientFileDescriptor_aint[i],returnVal_ptimeRequest,sizeRequest_int,0) /*read from socket*/
	  <sizeRequest_int){
         returnVal_ptimeRequest = NULL;
         freeDisconnectedClient(clientFileDescriptor_aint[i]);
      }
      else{
        returnVal_ptimeRequest->clientID =clientFileDescriptor_aint[i]; /*Assign file descriptor to client ID*/
	returnVal_ptimeRequest->reqType = ntohs(returnVal_ptimeRequest->reqType); /*convert data from network byte ordering*/

        break; /*end of processing, read OK data*/
     }

    }/*End of conditional on incoming data*/
  }/*end of loop*/

  errno = 0;
  return returnVal_ptimeRequest;

}
/*
function servicing
task: examine internal data structure to determine if we are currently serving requests. Pending connections are not examined and sockets are not polled for disconnections
input: none
output: TRUE if there are active clients, or if no clients have yet connected , FALSE otherwise
*/
boolean servicing(){
  if (!previousConnections) return TRUE;
  else return  hasClientsConnected();

}
/*
function forwardReply
task: send a reply to the client of the last request. This information is held internally and therefore all requests should be answered before polling for new ones
input: reply_sz: zero-terminated string: holds the reply to send to the client
output: -1 on error, 0 otherwise
*/
int forwardReply(const char * reply_sz){
  int reply_size = strlen(reply_sz);

/*Write to socket*/
  if (
      send (current_timeRequest.clientID, reply_sz, reply_size, 0)
      != reply_size) /*Error on sending*/
  return -1;
  else return 0;

}

/*
function shutDownNetwork
task: terminate all network connections
input: none
output: none
*/
void shutDownNetwork(){
  int i;
  for (i = 0; i <currentConnections; i++){
    close(clientFileDescriptor_aint[i]);
  }
  currentConnections = 0;
  close(acceptingSocket_int);
}
#ifdef DATASTRUCT_TEST
#include <stdio.h>
void printClientList(){
  int i;
  printf("List:\t");
  for (i = 0;i <currentConnections; i++){
    printf("%d ", clientFileDescriptor_aint[i]);
  }
  printf("\n");
}


int main(int argc, char** argv){
int i;
  for (i  = 0; i < 30; i++) addClient(i);
  printClientList();

  for(i = 30; i >=0; i--) {
    removeClient(i);
    printClientList();
  }

  for (i  = 0; i < 30; i++) addClient(i);
  printClientList();

  for(i  = 0; i < 30; i++) {
    removeClient(i);
    printClientList();
  }

  for (i  = 0; i < 30; i++) addClient(i);
  printClientList();

  for(i  = 15; i < 30; i++) {
    removeClient(i);
    printClientList();
  }


  return 0;
}

#endif

#ifdef NETWORK_TEST
#include <stdio.h>
void printClientList(){
  int i;
  printf("Connected clients' list:\t");
  for (i = 0;i <currentConnections; i++){
    printf("%d ", clientFileDescriptor_aint[i]);
  }
  printf("\n");
}
int main(int argc, char** argv){
  int mockClient = -1;
  struct sockaddr_in connectAddress_sockaddrIn;
  int i;
  struct timeRequest test_timeRequest;
  char replybuf_sz[1024];

  for (i =0; i <2; i++){
    WriteError("\n\n===============================================\n");
	if(setConnection() == -1){
		WriteError("Cannot initialize server port\n");
		exit(1);
	}
	else WriteError("Server port initialized\n");
	sleep(3);

	WriteError("Creating fake client and connecting it\n");
	if ((mockClient = socket(AF_INET, SOCK_STREAM,0)) == -1){
		WriteError("Cannot create client socket\n");
	}
	else{
		WriteError("Connecting fake client\n");
		/*Prepare bind-specific information From Donahoo/Calvert p.19*/
		memset(&connectAddress_sockaddrIn, 0, sizeof(struct sockaddr_in));
		connectAddress_sockaddrIn.sin_family = AF_INET;
		connectAddress_sockaddrIn.sin_addr.s_addr = inet_addr("127.0.0.1");
		connectAddress_sockaddrIn.sin_port = htons(SERVERPORT);

		if(connect(mockClient, (struct sockaddr*)&connectAddress_sockaddrIn, sizeof(connectAddress_sockaddrIn))<0){
			perror("connect ");
			WriteError("Could not connect client\n");
		}

		WriteError("Accepting client connection\n");
		AcceptNewConnection();
		printClientList();


		if (i == 1){
			WriteError("Testing mocked-up request sending and replying");
			test_timeRequest.reqType = TIME;
			
			/*Sending request*/
			send(mockClient,&test_timeRequest, sizeof(test_timeRequest),0);
			pollRequest();
			
			sprintf(replybuf_sz, "Request received ID: %.0f\n", (float) current_timeRequest.clientID);
			WriteError(replybuf_sz);
			
			
			forwardReply("a reply\n");

			recv(mockClient, replybuf_sz, 1024, 0);
			
			WriteError(replybuf_sz);



		}else WriteError("Testing connection management\n");


		WriteError("closing client\n");
		close (mockClient);

		WriteError("polling for disconnected client\n");
		pollRequest();
		printClientList();

		if(servicing()) WriteError("Error: Still some marked as connected\n");

	}
	WriteError("Closing accepting socket\n");
	close(acceptingSocket_int);

  }
  return 0;
}
#endif


