#ifndef TIMESERVER_TCP_H
#define TIMESERVER_TCP_H

#include "../common/shared.h"

/*
function AcceptNewConnection
task: poll listening socket for an incoming connection and accept it if there is one
input: none
output: -1 on error, 0 otherwise. 0 is returned wheter a client has been added or not.
*/
int AcceptNewConnection();

/*
function pollRequest
task: find the first socket in the internal list with a request in it. If a client has disconnected, it is removed from the list
input: none
output: a pointer to a struct timeRequest * if a request is in, NULL otherwise.
Furthermore, errno is set to the following:
  0: success
  EIO: error on polling on the socket
*/
struct timeRequest * pollRequest();

/*
function forwardReply
task: send a reply to the client of the last request. This information is held internally and therefore all requests should be answered before polling for new ones
input: reply_sz: zero-terminated string: holds the reply to send to the client
output: -1 on error, 0 otherwise
*/
int forwardReply(const char * reply_sz);

/*
function servicing
task: examine internal data structure to determine if we are currently serving requests. Pending connections are not examined and sockets are not polled for disconnections
input: none
output: TRUE if there are active clients, or if no clients have yet connected , FALSE otherwise
*/
boolean servicing();

/*
function shutDownNetwork
task: terminate all network connections
input: none
output: none
*/
void shutDownNetwork();

/*
function setConnection
task: initialize server socket
input:none
output: -1 on error, 0 otherwise
*/
int setConnection();
/*
function removeClient
task: disconnect socket and remove client from the list of connected clients
input: fileDescriptorToRemove_int: int: file descriptor of the client to disconnect
output: -1 on error, 0 otherwise
*/
int removeClient(int fileDescriptorToRemove_int);
#endif

